@@ -706,6 +706,8 @@ struct htaccess_result {
     int override;
     /** the override options allowed for the .htaccess file */
     int override_opts;
+    /** Table of allowed directives for override */
+    apr_table_t *override_list;
     /** the configuration directives */
     struct ap_conf_vector_t *htaccess;
     /** the next one, or NULL if no more; N.B. never change this */
@@ -1133,7 +1135,9 @@ typedef enum  {
     CONN_STATE_HANDLER,
     CONN_STATE_WRITE_COMPLETION,
     CONN_STATE_SUSPENDED,
-    CONN_STATE_LINGER
+    CONN_STATE_LINGER,
+    CONN_STATE_LINGER_NORMAL,
+    CONN_STATE_LINGER_SHORT
 } conn_state_e;
 
 /** 
@@ -1150,7 +1154,7 @@ struct conn_state_t {
     apr_pool_t *p;
     /** bucket allocator */
     apr_bucket_alloc_t *bucket_alloc;
-    /** poll file decriptor information */
+    /** poll file descriptor information */
     apr_pollfd_t pfd;
     /** Current state of the connection */
     conn_state_e state;
@@ -1295,6 +1299,42 @@ typedef struct core_net_rec {
     core_ctx_t *in_ctx;
 } core_net_rec;
 
+/**
+ * Get the context_document_root for a request. This is a generalization of
+ * the document root, which is too limited in the presence of mappers like
+ * mod_userdir and mod_alias. The context_document_root is the directory
+ * on disk that maps to the context_prefix URI prefix.
+ * @param r The request
+ * @note For resources that do not map to the file system or for very complex
+ * mappings, this information may still be wrong.
+ */
+AP_DECLARE(const char *) ap_context_document_root(request_rec *r);
+
+/**
+ * Get the context_prefix for a request. The context_prefix URI prefix
+ * maps to the context_document_root on disk.
+ * @param r The request
+ */
+AP_DECLARE(const char *) ap_context_prefix(request_rec *r);
+
+/** Set context_prefix and context_document_root for a request.
+ * @param r The request
+ * @param prefix the URI prefix, without trailing slash
+ * @param document_root the corresponding directory on disk, without trailing
+ * slash
+ * @note If one of prefix of document_root is NULL, the corrsponding
+ * property will not be changed.
+ */
+AP_DECLARE(void) ap_set_context_info(request_rec *r, const char *prefix,
+                                     const char *document_root);
+
+/** Set per-request document root. This is for mass virtual hosting modules
+ * that want to provide the correct DOCUMENT_ROOT value to scripts.
+ * @param r The request
+ * @param document_root the document root for the request.
+ */
+AP_DECLARE(void) ap_set_document_root(request_rec *r, const char *document_root);
+
 /**
  * Examine a field value (such as a media-/content-type) string and return
  * it sans any parameters; e.g., strip off any ';charset=foo' and the like.
@@ -1494,9 +1534,10 @@ AP_DECLARE(int) ap_unescape_url(char *url);
 /**
  * Unescape a URL, but leaving %2f (slashes) escaped
  * @param url The url to unescape
+ * @param decode_slashes Whether or not slashes should be decoded
  * @return 0 on success, non-zero otherwise
  */
-AP_DECLARE(int) ap_unescape_url_keep2f(char *url);
+AP_DECLARE(int) ap_unescape_url_keep2f(char *url, int decode_slashes);
 
 /**
  * Convert all double slashes to single slashes
@@ -1759,6 +1800,12 @@ AP_DECLARE(void) ap_content_type_tolower(char *s);
  */
 AP_DECLARE(void) ap_str_tolower(char *s);
 
+/**
+ * convert a string to all uppercase
+ * @param s The string to convert to uppercase 
+ */
+AP_DECLARE(void) ap_str_toupper(char *s);
+
 /**
  * Search a string from left to right for the first occurrence of a 
  * specific character
@@ -1936,7 +1983,7 @@ AP_DECLARE(void) ap_log_assert(const char *szExp, const char *szFile, int nLine)
 #endif
 
 /**
- * @defgroup stopsignal Flags which indicate places where the sever should stop for debugging.
+ * @defgroup stopsignal Flags which indicate places where the server should stop for debugging.
  * @{
  * A set of flags which indicate places where the server should raise(SIGSTOP).
  * This is useful for debugging, because you can then attach to that process
