@@ -39,6 +39,7 @@
 
 #include "ap_config.h"
 #include "ap_mpm.h"
+#include "scoreboard.h"
 
 #if APR_HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>    /* for TCP_NODELAY */
@@ -80,56 +81,60 @@ extern "C" {
 /* Signal used to gracefully stop (as a quoted string) */
 #define AP_SIG_GRACEFUL_STOP_STRING "SIGWINCH"
 
+/**
+ * Callback function used for ap_reclaim_child_processes() and
+ * ap_relieve_child_processes().  The callback function will be
+ * called for each terminated child process.
+ */
+typedef void ap_reclaim_callback_fn_t(int childnum, pid_t pid,
+                                      ap_generation_t gen);
+
 /**
  * Make sure all child processes that have been spawned by the parent process
  * have died.  This includes process registered as "other_children".
  * @param terminate Either 1 or 0.  If 1, send the child processes SIGTERM
  *        each time through the loop.  If 0, give the process time to die
  *        on its own before signalling it.
- * @note This function requires that a hook is implemented by the MPM: <pre>
- *  mpm_note_child_killed -- Note the child died in the scoreboard
- * </pre>
  *
  * @note The MPM child processes which are reclaimed are those listed
  * in the scoreboard as well as those currently registered via
  * ap_register_extra_mpm_process().
  */
-void ap_reclaim_child_processes(int terminate);
+void ap_reclaim_child_processes(int terminate,
+                                ap_reclaim_callback_fn_t *mpm_callback);
 
 /**
  * Catch any child processes that have been spawned by the parent process
  * which have exited. This includes processes registered as "other_children".
  *
- * @note This function requires that a hook is implemented by the MPM: <pre>
- *  mpm_note_child_killed -- Note the child died in the scoreboard
- * </pre>
- *
  * @note The MPM child processes which are relieved are those listed
  * in the scoreboard as well as those currently registered via
  * ap_register_extra_mpm_process().
  */
-void ap_relieve_child_processes(void);
+void ap_relieve_child_processes(ap_reclaim_callback_fn_t *mpm_callback);
 
 /**
  * Tell ap_reclaim_child_processes() and ap_relieve_child_processes() about 
  * an MPM child process which has no entry in the scoreboard.
  * @param pid The process id of an MPM child process which should be
  * reclaimed when ap_reclaim_child_processes() is called.
+ * @param gen The generation of this MPM child process.
  *
  * @note If an extra MPM child process terminates prior to calling
  * ap_reclaim_child_processes(), remove it from the list of such processes
  * by calling ap_unregister_extra_mpm_process().
  */
-void ap_register_extra_mpm_process(pid_t pid);
+void ap_register_extra_mpm_process(pid_t pid, ap_generation_t gen);
 
 /**
  * Unregister an MPM child process which was previously registered by a
  * call to ap_register_extra_mpm_process().
  * @param pid The process id of an MPM child process which no longer needs to
  * be reclaimed.
+ * @param old_gen Set to the server generation of the process, if found.
  * @return 1 if the process was found and removed, 0 otherwise
  */
-int ap_unregister_extra_mpm_process(pid_t pid);
+int ap_unregister_extra_mpm_process(pid_t pid, ap_generation_t *old_gen);
 
 /**
  * Safely signal an MPM child process, if the process is in the
@@ -143,9 +148,8 @@ int ap_unregister_extra_mpm_process(pid_t pid);
 apr_status_t ap_mpm_safe_kill(pid_t pid, int sig);
 
 /**
- * Determine if any child process has died.  If no child process died, then
- * this process sleeps for the amount of time specified by the MPM defined
- * macro SCOREBOARD_MAINTENANCE_INTERVAL.
+ * Run the monitor hook (once every ten calls), determine if any child
+ * process has died and, if none died, sleep one second.
  * @param status The return code if a process has died
  * @param exitcode The returned exit status of the child, if a child process 
  *                 dies, or the signal that caused the child to die.
@@ -199,6 +203,21 @@ AP_DECLARE(uid_t) ap_uname2id(const char *name);
 AP_DECLARE(gid_t) ap_gname2id(const char *name);
 #endif
 
+#ifndef HAVE_INITGROUPS
+/**
+ * The initgroups() function initializes the group access list by reading the
+ * group database /etc/group and using all groups of which user is a member.
+ * The additional group basegid is also added to the list. 
+ * @param name The user name - must be non-NULL
+ * @param basegid The basegid to add
+ * @return returns 0 on success
+ * @fn int initgroups(const char *name, gid_t basegid)
+ */
+int initgroups(const char *name, gid_t basegid);
+#endif
+
+#if !defined(WIN32) || defined(DOXYGEN)
+
 typedef struct ap_pod_t ap_pod_t;
 
 struct ap_pod_t {
@@ -242,6 +261,8 @@ AP_DECLARE(apr_status_t) ap_mpm_pod_signal(ap_pod_t *pod);
  */
 AP_DECLARE(void) ap_mpm_pod_killpg(ap_pod_t *pod, int num);
 
+#endif /* !WIN32 || DOXYGEN */
+
 /**
  * Check that exactly one MPM is loaded
  * Returns NULL if yes, error string if not.
@@ -304,13 +325,15 @@ extern const char *ap_mpm_set_thread_stacksize(cmd_parms *cmd, void *dummy,
 extern apr_status_t ap_fatal_signal_setup(server_rec *s, apr_pool_t *pconf);
 extern apr_status_t ap_fatal_signal_child_setup(server_rec *s);
 
+/* core's implementation of child_status hook */
+extern void ap_core_child_status(server_rec *s, pid_t pid, ap_generation_t gen,
+                                 int slot, mpm_child_status status);
+
 #if AP_ENABLE_EXCEPTION_HOOK
 extern const char *ap_mpm_set_exception_hook(cmd_parms *cmd, void *dummy,
                                              const char *arg);
 #endif
 
-AP_DECLARE(apr_status_t) ap_mpm_note_child_killed(int childnum);
-
 AP_DECLARE_HOOK(int,monitor,(apr_pool_t *p, server_rec *s))
 
 /* register modules that undertake to manage system security */
@@ -323,9 +346,6 @@ AP_DECLARE_HOOK(int, drop_privileges, (apr_pool_t * pchild, server_rec * s))
  */
 AP_DECLARE_HOOK(int, mpm_query, (int query_code, int *result, apr_status_t *rv))
 
-/* child specified by index has been killed */
-AP_DECLARE_HOOK(apr_status_t, mpm_note_child_killed, (int childnum))
-
 /* register the specified callback */
 AP_DECLARE_HOOK(apr_status_t, mpm_register_timed_callback,
                 (apr_time_t t, ap_mpm_callback_fn_t *cbfn, void *baton))
@@ -338,6 +358,11 @@ AP_DECLARE_HOOK(const char *,mpm_get_name,(void))
  */
 #define AP_ACCEPT_MUTEX_TYPE "mpm-accept"
 
+/* internal pre-config logic for MPM-related settings, callable only from
+ * core's pre-config hook
+ */
+void mpm_common_pre_config(apr_pool_t *pconf);
+  
 #ifdef __cplusplus
 }
 #endif
