@@ -1100,7 +1100,6 @@ static const char *gen_nonce(apr_pool_t *p, apr_time_t now, const char *opaque,
                              const digest_config_rec *conf)
 {
     char *nonce = apr_palloc(p, NONCE_LEN+1);
-    int len;
     time_rec t;
 
     if (conf->nonce_lifetime != 0) {
@@ -1116,7 +1115,7 @@ static const char *gen_nonce(apr_pool_t *p, apr_time_t now, const char *opaque,
         /* XXX: WHAT IS THIS CONSTANT? */
         t.time = 42;
     }
-    len = apr_base64_encode_binary(nonce, t.arr, sizeof(t.arr));
+    apr_base64_encode_binary(nonce, t.arr, sizeof(t.arr));
     gen_nonce_hash(nonce+NONCE_TIME_LEN, nonce, opaque, server, conf);
 
     return nonce;
@@ -1184,7 +1183,7 @@ static client_entry *gen_client(const request_rec *r)
  * contacting the password server with the necessary information (typically
  * the username, realm, nonce, and cnonce) and receiving the hash from it.
  *
- * TBD: This function should probably be in a seperate source file so that
+ * TBD: This function should probably be in a separate source file so that
  * people need not modify mod_auth_digest.c each time they install a new
  * version of apache.
  */
@@ -1260,10 +1259,8 @@ static void note_digest_auth_failure(request_rec *r,
                                      digest_header_rec *resp, int stale)
 {
     const char   *qop, *opaque, *opaque_param, *domain, *nonce;
-    int           cnt;
 
     /* Setup qop */
-
     if (conf->qop_list[0] == NULL) {
         qop = ", qop=\"auth\"";
     }
@@ -1271,6 +1268,7 @@ static void note_digest_auth_failure(request_rec *r,
         qop = "";
     }
     else {
+        int cnt;
         qop = apr_pstrcat(r->pool, ", qop=\"", conf->qop_list[0], NULL);
         for (cnt = 1; conf->qop_list[cnt] != NULL; cnt++) {
             qop = apr_pstrcat(r->pool, qop, ",", conf->qop_list[cnt], NULL);
@@ -1340,7 +1338,7 @@ static void note_digest_auth_failure(request_rec *r,
 
     /* don't send domain
      * - for proxy requests
-     * - if it's no specified
+     * - if it's not specified
      */
     if (r->proxyreq || !conf->uri_list) {
         domain = NULL;
@@ -1516,7 +1514,6 @@ static int check_nonce(request_rec *r, digest_header_rec *resp,
                        const digest_config_rec *conf)
 {
     apr_time_t dt;
-    int len;
     time_rec nonce_time;
     char tmp, hash[NONCE_HASH_LEN+1];
 
@@ -1530,7 +1527,7 @@ static int check_nonce(request_rec *r, digest_header_rec *resp,
 
     tmp = resp->nonce[NONCE_TIME_LEN];
     resp->nonce[NONCE_TIME_LEN] = '\0';
-    len = apr_base64_decode_binary(nonce_time.arr, resp->nonce);
+    apr_base64_decode_binary(nonce_time.arr, resp->nonce);
     gen_nonce_hash(hash, resp->nonce, resp->opaque, r->server, conf);
     resp->nonce[NONCE_TIME_LEN] = tmp;
     resp->nonce_time = nonce_time.time;
