@@ -843,6 +843,9 @@ static int cache_save_filter(ap_filter_t *f, apr_bucket_brigade *in)
 
             r->headers_out = cache->stale_handle->resp_hdrs;
 
+            ap_set_content_type(r, apr_table_get(
+                    cache->stale_handle->resp_hdrs, "Content-Type"));
+
             /* add a revalidation warning */
             warn_head = apr_table_get(r->err_headers_out, "Warning");
             if ((warn_head == NULL) || ((warn_head != NULL)
@@ -967,11 +970,18 @@ static int cache_save_filter(ap_filter_t *f, apr_bucket_brigade *in)
         reason = apr_pstrcat(p, "Broken expires header: ", exps, NULL);
     }
     else if (!dconf->store_expired && exp != APR_DATE_BAD
-            && exp < r->request_time)
-    {
+            && exp < r->request_time) {
         /* if a Expires header is in the past, don't cache it */
         reason = "Expires header already expired; not cacheable";
     }
+    else if (!dconf->store_expired && (control.must_revalidate
+            || control.proxy_revalidate) && (!control.s_maxage_value
+            || (!control.s_maxage && !control.max_age_value)) && lastmods
+            == NULL && etag == NULL) {
+        /* if we're already stale, but can never revalidate, don't cache it */
+        reason
+                = "s-maxage or max-age zero and no Last-Modified or Etag; not cacheable";
+    }
     else if (!conf->ignorequerystring && r->parsed_uri.query && exps == NULL
             && !control.max_age && !control.s_maxage) {
         /* if a query string is present but no explicit expiration time,
@@ -1523,10 +1533,27 @@ static int cache_remove_url_filter(ap_filter_t *f, apr_bucket_brigade *in)
  */
 static int cache_filter(ap_filter_t *f, apr_bucket_brigade *in)
 {
+
+    cache_server_conf
+            *conf =
+                    (cache_server_conf *) ap_get_module_config(f->r->server->module_config,
+                            &cache_module);
+
+    /* was the quick handler enabled */
+    if (conf->quick) {
+        ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, f->r,
+                "cache: CACHE filter was added in quick handler mode and "
+                "will be ignored: %s", f->r->unparsed_uri);
+    }
+    /* otherwise we may have been bypassed, nothing to see here */
+    else {
+        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, f->r,
+                "cache: CACHE filter was added twice, or was added where "
+                "the cache has been bypassed and will be ignored: %s",
+                f->r->unparsed_uri);
+    }
+
     /* we are just a marker, so let's just remove ourselves */
-    ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, f->r,
-            "cache: CACHE filter was added twice, or was added in quick "
-            "handler mode and will be ignored.");
     ap_remove_output_filter(f);
     return ap_pass_brigade(f->next, in);
 }
@@ -1720,6 +1747,9 @@ static void *create_dir_config(apr_pool_t *p, char *dummy)
 
     dconf->stale_on_error = DEFAULT_CACHE_STALE_ON_ERROR;
 
+    /* array of providers for this URL space */
+    dconf->cacheenable = apr_array_make(p, 10, sizeof(struct cache_enable));
+
     return dconf;
 }
 
@@ -1764,6 +1794,12 @@ static void *merge_dir_config(apr_pool_t *p, void *basev, void *addv) {
     new->stale_on_error_set = add->stale_on_error_set
             || base->stale_on_error_set;
 
+    new->cacheenable = add->enable_set ? apr_array_append(p, base->cacheenable,
+            add->cacheenable) : base->cacheenable;
+    new->enable_set = add->enable_set || base->enable_set;
+    new->disable = (add->disable_set == 0) ? base->disable : add->disable;
+    new->disable_set = add->disable_set || base->disable_set;
+
     return new;
 }
 
@@ -1993,6 +2029,7 @@ static const char *add_cache_enable(cmd_parms *parms, void *dummy,
                                     const char *type,
                                     const char *url)
 {
+    cache_dir_conf *dconf = (cache_dir_conf *)dummy;
     cache_server_conf *conf;
     struct cache_enable *new;
 
@@ -2023,7 +2060,15 @@ static const char *add_cache_enable(cmd_parms *parms, void *dummy,
     conf =
         (cache_server_conf *)ap_get_module_config(parms->server->module_config,
                                                   &cache_module);
-    new = apr_array_push(conf->cacheenable);
+
+    if (parms->path) {
+        new = apr_array_push(dconf->cacheenable);
+        dconf->enable_set = 1;
+    }
+    else {
+        new = apr_array_push(conf->cacheenable);
+    }
+
     new->type = type;
     if (apr_uri_parse(parms->pool, url, &(new->url))) {
         return NULL;
@@ -2040,6 +2085,7 @@ static const char *add_cache_enable(cmd_parms *parms, void *dummy,
 static const char *add_cache_disable(cmd_parms *parms, void *dummy,
                                      const char *url)
 {
+    cache_dir_conf *dconf = (cache_dir_conf *)dummy;
     cache_server_conf *conf;
     struct cache_disable *new;
 
@@ -2065,6 +2111,13 @@ static const char *add_cache_disable(cmd_parms *parms, void *dummy,
     conf =
         (cache_server_conf *)ap_get_module_config(parms->server->module_config,
                                                   &cache_module);
+
+    if (parms->path) {
+        dconf->disable = 1;
+        dconf->disable_set = 1;
+        return NULL;
+    }
+
     new = apr_array_push(conf->cachedisable);
     if (apr_uri_parse(parms->pool, url, &(new->url))) {
         return NULL;
