@@ -39,7 +39,10 @@
 
 #include "mod_dav.h"
 #include "repos.h"
+#include "http_log.h"
+#include "http_main.h"      /* for ap_server_conf */
 
+APLOG_USE_MODULE(dav_fs);
 
 struct dav_db {
     apr_pool_t *pool;
@@ -93,6 +96,9 @@ static dav_error * dav_fs_dbm_error(dav_db *db, apr_pool_t *p,
     if (db == NULL) {
         errcode = 1;
         errstr = "Could not open property database.";
+        if (APR_STATUS_IS_EDSOOPEN(status))
+            ap_log_error(APLOG_MARK, APLOG_CRIT, status, ap_server_conf,
+            "The DBM driver could not be loaded");
     }
     else {
         (void) apr_dbm_geterror(db->file, &errcode, errbuf, sizeof(errbuf));
@@ -122,7 +128,7 @@ dav_error * dav_dbm_open_direct(apr_pool_t *p, const char *pathname, int ro,
                                 dav_db **pdb)
 {
     apr_status_t status;
-    apr_dbm_t *file;
+    apr_dbm_t *file = NULL;
 
     *pdb = NULL;
 
@@ -487,7 +493,9 @@ static void dav_propdb_close(dav_db *db)
         memcpy(db->ns_table.buf, &m, sizeof(m));
 
         err = dav_dbm_store(db, key, value);
-        /* ### what to do with the error? */
+        if (err != NULL)
+            ap_log_error(APLOG_MARK, APLOG_WARNING, err->aprerr,
+                         ap_server_conf, "Error writing propdb: %s", err->desc);
     }
 
     dav_dbm_close(db);
