@@ -186,7 +186,7 @@ static apr_status_t reqtimeout_filter(ap_filter_t *f,
     }
 
     if (!ccfg->socket) {
-        ccfg->socket = ap_get_module_config(f->c->conn_config, &core_module);
+        ccfg->socket = ap_get_conn_socket(f->c);
     }
 
     rv = check_time_left(ccfg, &time_left);
@@ -306,6 +306,13 @@ out:
          * 2s (SECONDS_TO_LINGER).
          */
         apr_table_setn(f->c->notes, "short-lingering-close", "1");
+
+        /*
+         * Also, we must not allow keep-alive requests, as
+         * ap_finalize_protocol() may ignore our error status (if the timeout
+         * happened on a request body that is discarded).
+         */
+        f->c->keepalive = AP_CONN_CLOSE;
     }
     return rv;
 }
