@@ -223,7 +223,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     const apr_array_header_t *arr = apr_table_elts(r->subprocess_env);
     const apr_table_entry_t *elts = (const apr_table_entry_t *)arr->elts;
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+    ap_log_error(APLOG_MARK, APLOG_TRACE8, 0, r->server,
                          "Into ajp_marshal_into_msgb");
 
     if ((method = sc_for_req_method_by_id(r)) == UNKNOWN_METHOD) {
@@ -289,7 +289,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
                    "Error appending the header value");
             return AJP_EOVERFLOW;
         }
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+        ap_log_error(APLOG_MARK, APLOG_TRACE5, 0, r->server,
                    "ajp_marshal_into_msgb: Header[%d] [%s] = [%s]",
                    i, elts[i].key, elts[i].val);
     }
@@ -449,7 +449,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
         return AJP_EOVERFLOW;
     }
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+    ap_log_error(APLOG_MARK, APLOG_TRACE8, 0, r->server,
             "ajp_marshal_into_msgb: Done");
     return APR_SUCCESS;
 }
@@ -515,7 +515,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
         r->status_line = NULL;
     }
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+    ap_log_error(APLOG_MARK, APLOG_TRACE4, 0, r->server,
            "ajp_unmarshal_response: status = %d", status);
 
     rc = ajp_msg_get_uint16(msg, &num_headers);
@@ -536,7 +536,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
         num_headers = 0;
     }
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+    ap_log_error(APLOG_MARK, APLOG_TRACE4, 0, r->server,
            "ajp_unmarshal_response: Number of headers is = %d",
            num_headers);
 
@@ -594,7 +594,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
         }
 
         ap_xlate_proto_from_ascii(value, strlen(value));
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+        ap_log_error(APLOG_MARK, APLOG_TRACE5, 0, r->server,
                "ajp_unmarshal_response: Header[%d] [%s] = [%s]",
                        i, stringname, value);
 
@@ -604,8 +604,8 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
         if (strcasecmp(stringname, "Content-Type") == 0) {
              /* add corresponding filter */
             ap_set_content_type(r, apr_pstrdup(r->pool, value));
-            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-               "ajp_unmarshal_response: ap_set_content_type done");
+            ap_log_error(APLOG_MARK, APLOG_TRACE5, 0, r->server,
+               "ajp_unmarshal_response: ap_set_content_type to '%s'", value);
         }
     }
 
@@ -638,6 +638,7 @@ apr_status_t ajp_send_header(apr_socket_t *sock,
     }
 
     rc = ajp_ilink_send(sock, msg);
+    ajp_msg_log(r, msg, "ajp_send_header: ajp_ilink_send packet dump");
     if (rc != APR_SUCCESS) {
         ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
                "ajp_send_header: ajp_ilink_send failed");
@@ -681,9 +682,11 @@ apr_status_t ajp_read_header(apr_socket_t *sock,
                "ajp_read_header: ajp_ilink_receive failed");
         return rc;
     }
+    ajp_msg_log(r, *msg, "ajp_read_header: ajp_ilink_receive packet dump");
     rc = ajp_msg_peek_uint8(*msg, &result);
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-               "ajp_read_header: ajp_ilink_received %02x", result);
+    ap_log_error(APLOG_MARK, APLOG_TRACE1, 0, r->server,
+               "ajp_read_header: ajp_ilink_received %s (0x%02x)",
+               ajp_type_str(result), result);
     return APR_SUCCESS;
 }
 
@@ -692,8 +695,9 @@ int ajp_parse_type(request_rec  *r, ajp_msg_t *msg)
 {
     apr_byte_t result;
     ajp_msg_peek_uint8(msg, &result);
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-               "ajp_parse_type: got %02x", result);
+    ap_log_error(APLOG_MARK, APLOG_TRACE6, 0, r->server,
+               "ajp_parse_type: got %s (0x%02x)",
+               ajp_type_str(result), result);
     return (int) result;
 }
 
@@ -712,7 +716,9 @@ apr_status_t ajp_parse_header(request_rec  *r, proxy_dir_conf *conf,
     }
     if (result != CMD_AJP13_SEND_HEADERS) {
         ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-               "ajp_parse_headers: wrong type %02x expecting 0x04", result);
+               "ajp_parse_headers: wrong type %s (0x%02x) expecting %s (0x%02x)",
+               ajp_type_str(result), result,
+               ajp_type_str(CMD_AJP13_SEND_HEADERS), CMD_AJP13_SEND_HEADERS);
         return AJP_EBAD_HEADER;
     }
     return ajp_unmarshal_response(msg, r, conf);
@@ -734,7 +740,9 @@ apr_status_t  ajp_parse_data(request_rec  *r, ajp_msg_t *msg,
     }
     if (result != CMD_AJP13_SEND_BODY_CHUNK) {
         ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-               "ajp_parse_data: wrong type %02x expecting 0x03", result);
+               "ajp_parse_data: wrong type %s (0x%02x) expecting %s (0x%02x)",
+               ajp_type_str(result), result,
+               ajp_type_str(CMD_AJP13_SEND_BODY_CHUNK), CMD_AJP13_SEND_BODY_CHUNK);
         return AJP_EBAD_HEADER;
     }
     rc = ajp_msg_get_uint16(msg, len);
@@ -762,6 +770,29 @@ apr_status_t  ajp_parse_data(request_rec  *r, ajp_msg_t *msg,
     return APR_SUCCESS;
 }
 
+/* Check the reuse flag in CMD_AJP13_END_RESPONSE */
+apr_status_t ajp_parse_reuse(request_rec *r, ajp_msg_t *msg,
+                             apr_byte_t *reuse)
+{
+    apr_byte_t result;
+    apr_status_t rc;
+
+    rc = ajp_msg_get_uint8(msg, &result);
+    if (rc != APR_SUCCESS) {
+        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+               "ajp_parse_reuse: ajp_msg_get_byte failed");
+        return rc;
+    }
+    if (result != CMD_AJP13_END_RESPONSE) {
+        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+               "ajp_parse_reuse: wrong type %s (0x%02x) expecting %s (0x%02x)",
+               ajp_type_str(result), result,
+               ajp_type_str(CMD_AJP13_END_RESPONSE), CMD_AJP13_END_RESPONSE);
+        return AJP_EBAD_HEADER;
+    }
+    return ajp_msg_get_uint8(msg, reuse);
+}
+
 /*
  * Allocate a msg to send data
  */
