@@ -20,7 +20,7 @@
 #include "scoreboard.h"
 #include "ap_mpm.h"
 #include "apr_version.h"
-#include "apr_hooks.h"
+#include "ap_hooks.h"
 #include "apr_date.h"
 
 static const char *balancer_mutex_type = "proxy-balancer-shm";
@@ -149,7 +149,7 @@ static char *get_path_param(apr_pool_t *pool, char *url,
         path += strlen(name);
         if (*path == '=') {
             /*
-             * Session path was found, get it's value
+             * Session path was found, get its value
              */
             ++path;
             if (*path) {
@@ -180,7 +180,7 @@ static char *get_cookie_param(request_rec *r, const char *name)
                     ++start_cookie;
                 if (*start_cookie++ == '=' && *start_cookie) {
                     /*
-                     * Session cookie was found, get it's value
+                     * Session cookie was found, get its value
                      */
                     char *end_cookie, *cookie;
                     cookie = apr_pstrdup(r->pool, start_cookie);
@@ -654,7 +654,7 @@ static void recalc_factors(proxy_balancer *balancer)
 
     /* Recalculate lbfactors */
     workers = (proxy_worker **)balancer->workers->elts;
-    /* Special case if there is only one worker it's
+    /* Special case if there is only one worker its
      * load factor will always be 1
      */
     if (balancer->workers->nelts == 1) {
@@ -703,10 +703,12 @@ static int balancer_post_config(apr_pool_t *pconf, apr_pool_t *plog,
     /*
      * Get slotmem setups
      */
-    storage = ap_lookup_provider(AP_SLOTMEM_PROVIDER_GROUP, "shared", "0");
+    storage = ap_lookup_provider(AP_SLOTMEM_PROVIDER_GROUP, "shared",
+                                 AP_SLOTMEM_PROVIDER_VERSION);
     if (!storage) {
         ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_EMERG, 0, s,
-                     "ap_lookup_provider %s failed", AP_SLOTMEM_PROVIDER_GROUP);
+                     "ap_lookup_provider %s failed: is mod_slotmem_shm loaded??",
+                     AP_SLOTMEM_PROVIDER_GROUP);
         return !OK;
     }
 
@@ -722,13 +724,14 @@ static int balancer_post_config(apr_pool_t *pconf, apr_pool_t *plog,
         conf = (proxy_server_conf *)ap_get_module_config(sconf, &proxy_module);
 
         if (conf->balancers->nelts) {
-            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, "Doing balancers create: %d, %d",
+            conf->max_balancers = conf->balancers->nelts + conf->bgrowth;
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, "Doing balancers create: %d, %d (%d)",
                          (int)ALIGNED_PROXY_BALANCER_SHARED_SIZE,
-                         (int)conf->balancers->nelts);
+                         (int)conf->balancers->nelts, conf->max_balancers);
 
             rv = storage->create(&new, conf->id,
                                  ALIGNED_PROXY_BALANCER_SHARED_SIZE,
-                                 conf->balancers->nelts, AP_SLOTMEM_TYPE_PREGRAB, pconf);
+                                 conf->max_balancers, AP_SLOTMEM_TYPE_PREGRAB, pconf);
             if (rv != APR_SUCCESS) {
                 ap_log_error(APLOG_MARK, APLOG_EMERG, rv, s, "balancer slotmem_create failed");
                 return !OK;
@@ -977,6 +980,8 @@ static int balancer_handler(request_rec *r)
     /* First set the params */
     if (wsel && ok2change) {
         const char *val;
+        int was_usable = PROXY_WORKER_IS_USABLE(wsel);
+
         ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server, "settings worker params");
 
         if ((val = apr_table_get(params, "w_lf"))) {
@@ -1017,6 +1022,10 @@ static int balancer_handler(request_rec *r)
                 wsel->s->lbset = ival;
              }
         }
+        /* if enabling, we need to reset all lb params */
+        if (bsel && !was_usable && PROXY_WORKER_IS_USABLE(wsel)) {
+            bsel->s->need_reset = 1;
+        }
 
     }
 
@@ -1025,13 +1034,15 @@ static int balancer_handler(request_rec *r)
         int ival;
         ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server, "settings balancer params");
         if ((val = apr_table_get(params, "b_lbm"))) {
-            if (strlen(val) < (sizeof(bsel->s->lbpname)-1)) {
+            if ((strlen(val) < (sizeof(bsel->s->lbpname)-1)) &&
+                strcmp(val, bsel->s->lbpname)) {
                 proxy_balancer_method *lbmethod;
-                lbmethod = ap_lookup_provider(PROXY_LBMETHOD, bsel->s->lbpname, "0");
+                lbmethod = ap_lookup_provider(PROXY_LBMETHOD, val, "0");
                 if (lbmethod) {
                     PROXY_STRNCPY(bsel->s->lbpname, val);
                     bsel->lbmethod = lbmethod;
                     bsel->s->wupdated = apr_time_now();
+                    bsel->s->need_reset = 1;
                 }
             }
         }
@@ -1219,7 +1230,7 @@ static int balancer_handler(request_rec *r)
                 "<th>Worker URL</th>"
                 "<th>Route</th><th>RouteRedir</th>"
                 "<th>Factor</th><th>Set</th><th align='center'>Status</th>"
-                "<th>Elected</th><th>To</th><th>From</th>"
+                "<th>Elected</th><th>Busy</th><th>Load</th><th>To</th><th>From</th>"
                 "</tr>\n", r);
 
             workers = (proxy_worker **)balancer->workers->elts;
@@ -1240,7 +1251,9 @@ static int balancer_handler(request_rec *r)
                 ap_rprintf(r, "<td align='center'>%d</td><td align='center'>", worker->s->lbset);
                 ap_rvputs(r, ap_proxy_parse_wstatus(r->pool, worker), NULL);
                 ap_rputs("</td>", r);
-                ap_rprintf(r, "<td align='center'>%" APR_SIZE_T_FMT "</td><td align='center'>", worker->s->elected);
+                ap_rprintf(r, "<td align='center'>%" APR_SIZE_T_FMT "</td>", worker->s->elected);
+                ap_rprintf(r, "<td align='center'>%" APR_SIZE_T_FMT "</td>", worker->s->busy);
+                ap_rprintf(r, "<td align='center'>%d</td><td align='center'>", worker->s->lbstatus);
                 ap_rputs(apr_strfsize(worker->s->transferred, fbuf), r);
                 ap_rputs("</td><td align='center'>", r);
                 ap_rputs(apr_strfsize(worker->s->read, fbuf), r);
