@@ -57,6 +57,7 @@ typedef struct {
 
 struct ap_slotmem_instance_t {
     char                 *name;       /* per segment name */
+    int                  fbased;      /* filebased? */
     void                 *shm;        /* ptr to memory segment (apr_shm_t *) */
     void                 *base;       /* data set start */
     apr_pool_t           *gpool;      /* per segment global pool */
@@ -207,20 +208,19 @@ static apr_status_t cleanup_slotmem(void *param)
 
     if (*mem) {
         ap_slotmem_instance_t *next = *mem;
-        apr_pool_t *p = next->gpool;
         while (next) {
             if (AP_SLOTMEM_IS_PERSIST(next)) {
                 store_slotmem(next);
             }
+            if (next->fbased) {
+                apr_shm_remove(next->name, next->gpool);
+            }
             apr_shm_destroy((apr_shm_t *)next->shm);
             next = next->next;
         }
-        apr_pool_destroy(p);
-    } else {
-        /* If shared mem was never called, then just remove
-         * the global pool */
-        apr_pool_destroy(gpool);
     }
+    /* apr_pool_destroy(gpool); */
+    globallistmem = NULL;
     return APR_SUCCESS;
 }
 
@@ -257,6 +257,7 @@ static apr_status_t slotmem_create(ap_slotmem_instance_t **new,
                                    ap_slotmem_type_t type, apr_pool_t *pool)
 {
 /*    void *slotmem = NULL; */
+    int fbased;
     char *ptr;
     sharedslotdesc_t desc;
     ap_slotmem_instance_t *res;
@@ -298,7 +299,8 @@ static apr_status_t slotmem_create(ap_slotmem_instance_t **new,
     }
 
     /* first try to attach to existing shared memory */
-    if (name && name[0] != ':') {
+    fbased = (name && name[0] != ':');
+    if (fbased) {
         rv = apr_shm_attach(&shm, fname, gpool);
     }
     else {
@@ -320,7 +322,7 @@ static apr_status_t slotmem_create(ap_slotmem_instance_t **new,
     }
     else {
         apr_size_t dsize = size - AP_SLOTMEM_OFFSET;
-        if (name && name[0] != ':') {
+        if (fbased) {
             apr_shm_remove(fname, gpool);
             rv = apr_shm_create(&shm, size, fname, gpool);
         }
@@ -330,7 +332,7 @@ static apr_status_t slotmem_create(ap_slotmem_instance_t **new,
         if (rv != APR_SUCCESS) {
             return rv;
         }
-        if (name && name[0] != ':') {
+        if (fbased) {
             /* Set permissions to shared memory
              * so it can be attached by child process
              * having different user credentials
@@ -359,6 +361,7 @@ static apr_status_t slotmem_create(ap_slotmem_instance_t **new,
     res = (ap_slotmem_instance_t *) apr_pcalloc(gpool,
                                                 sizeof(ap_slotmem_instance_t));
     res->name = apr_pstrdup(gpool, fname);
+    res->fbased = fbased;
     res->shm = shm;
     res->num_free = (unsigned int *)ptr;
     *res->num_free = item_num;
@@ -440,6 +443,7 @@ static apr_status_t slotmem_attach(ap_slotmem_instance_t **new,
     res = (ap_slotmem_instance_t *) apr_pcalloc(gpool,
                                                 sizeof(ap_slotmem_instance_t));
     res->name = apr_pstrdup(gpool, fname);
+    res->fbased = 1;
     res->shm = shm;
     res->num_free = (unsigned int *)ptr;
     res->persist = (void *)ptr;
@@ -572,7 +576,7 @@ static apr_status_t slotmem_grab(ap_slotmem_instance_t *slot, unsigned int *id)
         }
     }
     if (i >= slot->desc.num) {
-        return APR_ENOSHMAVAIL;
+        return APR_EINVAL;
     }
     *inuse = 1;
     *id = i;
@@ -646,23 +650,15 @@ static int post_config(apr_pool_t *p, apr_pool_t *plog, apr_pool_t *ptemp,
 static int pre_config(apr_pool_t *p, apr_pool_t *plog,
                       apr_pool_t *ptemp)
 {
-    apr_pool_t *global_pool;
-    apr_status_t rv;
-
-    rv = apr_pool_create(&global_pool, NULL);
-    if (rv != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_CRIT, rv, NULL,
-            "Fatal error: unable to create global pool for shared slotmem");
-        return rv;
-    }
-    slotmem_shm_initgpool(global_pool);
+    slotmem_shm_initgpool(p);
     return OK;
 }
 
 static void ap_slotmem_shm_register_hook(apr_pool_t *p)
 {
     const ap_slotmem_provider_t *storage = slotmem_shm_getstorage();
-    ap_register_provider(p, AP_SLOTMEM_PROVIDER_GROUP, "shared", "0", storage);
+    ap_register_provider(p, AP_SLOTMEM_PROVIDER_GROUP, "shared",
+                         AP_SLOTMEM_PROVIDER_VERSION, storage);
     ap_hook_post_config(post_config, NULL, NULL, APR_HOOK_LAST);
     ap_hook_pre_config(pre_config, NULL, NULL, APR_HOOK_MIDDLE);
 }
