@@ -178,11 +178,14 @@ void ssl_pphrase_Handle(server_rec *s, apr_pool_t *p)
      */
     for (pServ = s; pServ != NULL; pServ = pServ->next) {
         sc = mySrvConfig(pServ);
-
-        if (!sc->enabled)
-            continue;
-
         cpVHostID = ssl_util_vhostid(p, pServ);
+        if (!sc->enabled) {
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, pServ,
+                         "SSL not enabled on vhost %s, skipping SSL setup",
+                         cpVHostID);
+            continue;
+        }
+
         ap_log_error(APLOG_MARK, APLOG_INFO, 0, pServ,
                      "Loading certificate & private key of SSL-aware server '%s'",
                      cpVHostID);
@@ -206,7 +209,7 @@ void ssl_pphrase_Handle(server_rec *s, apr_pool_t *p)
         algoKey  = SSL_ALGO_UNKNOWN;
 
         /* Iterate through configured certificate files for this
-         * cert. */
+         * server. */
         for (i = 0, j = 0; i < SSL_AIDX_MAX
                  && (sc->server->pks->cert_files[i] != NULL
                      || sc->server->pkcs7); i++) {
@@ -235,6 +238,9 @@ void ssl_pphrase_Handle(server_rec *s, apr_pool_t *p)
                     ssl_log_ssl_error(SSLLOG_MARK, APLOG_EMERG, s);
                     ssl_die();
                 }
+                ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
+                             "Init: Read server certificate from '%s'",
+                             szPath);
             }
             /*
              * check algorithm type of certificate and make
@@ -452,8 +458,9 @@ void ssl_pphrase_Handle(server_rec *s, apr_pool_t *p)
                     }
                 }
                 else {
-                    ap_log_error(APLOG_MARK, APLOG_EMERG, 0,
-                                 pServ, "Init: Pass phrase incorrect");
+                    ap_log_error(APLOG_MARK, APLOG_EMERG, 0, pServ,
+                                 "Init: Pass phrase incorrect for key of %s",
+                                 cpVHostID);
                     ssl_log_ssl_error(SSLLOG_MARK, APLOG_EMERG, pServ);
 
                     if (writetty) {
@@ -747,8 +754,8 @@ int ssl_pphrase_Handle_CB(char *buf, int bufsize, int verify, void *srv)
          * something...
          */
         if (*pnPassPhraseDialog == 1) {
-            apr_file_printf(writetty, "%s mod_ssl/%s (Pass Phrase Dialog)\n",
-                            AP_SERVER_BASEVERSION, MOD_SSL_VERSION);
+            apr_file_printf(writetty, "%s mod_ssl (Pass Phrase Dialog)\n",
+                            AP_SERVER_BASEVERSION);
             apr_file_printf(writetty, "Some of your private key files are encrypted for security reasons.\n");
             apr_file_printf(writetty, "In order to read them you have to provide the pass phrases.\n");
         }
@@ -813,7 +820,7 @@ int ssl_pphrase_Handle_CB(char *buf, int bufsize, int verify, void *srv)
     *cppPassPhraseCur = apr_pstrdup(p, buf);
 
     /*
-     * And return it's length to OpenSSL...
+     * And return its length to OpenSSL...
      */
     return (len);
 }
