@@ -43,6 +43,7 @@
 #include "apr.h"
 #include "apr_strings.h"
 #define APR_WANT_STRFUNC
+#define APR_WANT_MEMFUNC
 #include "apr_want.h"
 #include "apr_tables.h"
 #include "apr_lib.h"
@@ -53,7 +54,9 @@
 #include "ap_socache.h"
 #include "mod_auth.h"
 
-#define MOD_SSL_VERSION AP_SERVER_BASEREVISION
+#ifdef APR_HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 
 #ifndef FALSE
 #define FALSE 0
@@ -63,27 +66,10 @@
 #define TRUE !FALSE
 #endif
 
-#ifndef YY_NULL
-#define YY_NULL 0
-#endif
-
-#ifndef MIN
-#define MIN(a,b) (((a)<(b))?(a):(b))
-#endif
-
 #ifndef BOOL
 #define BOOL unsigned int
 #endif
 
-#ifndef NULL
-#define NULL (void *)0
-#endif
-
-#ifndef NUL
-#define NUL '\0'
-#endif
-
-
 /* mod_ssl headers */
 #include "ssl_toolkit_compat.h"
 #include "ap_expr.h"
@@ -250,7 +236,11 @@ typedef int ssl_opt_t;
 #define SSL_PROTOCOL_SSLV2 (1<<0)
 #define SSL_PROTOCOL_SSLV3 (1<<1)
 #define SSL_PROTOCOL_TLSV1 (1<<2)
+#ifndef OPENSSL_NO_SSL2
 #define SSL_PROTOCOL_ALL   (SSL_PROTOCOL_SSLV2|SSL_PROTOCOL_SSLV3|SSL_PROTOCOL_TLSV1)
+#else
+#define SSL_PROTOCOL_ALL   (SSL_PROTOCOL_SSLV3|SSL_PROTOCOL_TLSV1)
+#endif
 typedef int ssl_proto_t;
 
 /**
