@@ -25,7 +25,6 @@
 #include "apr_fnmatch.h"
 #include "apr_hash.h"
 #include "apr_thread_proc.h"    /* for RLIMIT stuff */
-#include "apr_hooks.h"
 
 #define APR_WANT_IOVEC
 #define APR_WANT_STRFUNC
@@ -75,7 +74,9 @@ do { \
     } while (!APR_BRIGADE_EMPTY(b) && (e != APR_BRIGADE_SENTINEL(b))); \
 } while (0)
 
-APLOG_USE_MODULE(core);
+/* we know core's module_index is 0 */
+#undef APLOG_MODULE_INDEX
+#define APLOG_MODULE_INDEX AP_CORE_MODULE_INDEX
 
 int ap_core_input_filter(ap_filter_t *f, apr_bucket_brigade *b,
                          ap_input_mode_t mode, apr_read_type_e block,
@@ -599,7 +600,6 @@ static apr_status_t send_brigade_nonblocking(apr_socket_t *s,
     for (bucket = APR_BRIGADE_FIRST(bb);
          bucket != APR_BRIGADE_SENTINEL(bb);
          bucket = next) {
-        int did_sendfile = 0;
         next = APR_BUCKET_NEXT(bucket);
 #if APR_HAS_SENDFILE
         if (APR_BUCKET_IS_FILE(bucket)) {
@@ -613,7 +613,6 @@ static apr_status_t send_brigade_nonblocking(apr_socket_t *s,
 
             if ((apr_file_flags_get(fd) & APR_SENDFILE_ENABLED) &&
                 (bucket->length >= AP_MIN_SENDFILE_BYTES)) {
-                did_sendfile = 1;
                 if (nvec > 0) {
                     (void)apr_socket_opt_set(s, APR_TCP_NOPUSH, 1);
                     rv = writev_nonblocking(s, vec, nvec, bb, bytes_written, c);
@@ -634,7 +633,8 @@ static apr_status_t send_brigade_nonblocking(apr_socket_t *s,
             }
         }
 #endif /* APR_HAS_SENDFILE */
-        if (!did_sendfile && !APR_BUCKET_IS_METADATA(bucket)) {
+        /* didn't sendfile */
+        if (!APR_BUCKET_IS_METADATA(bucket)) {
             const char *data;
             apr_size_t length;
             rv = apr_bucket_read(bucket, &data, &length, APR_BLOCK_READ);
