@@ -54,7 +54,14 @@
 #include "util_time.h"
 #include "ap_mpm.h"
 
-APLOG_USE_MODULE(core);
+#if HAVE_GETTID
+#include <sys/syscall.h>
+#include <sys/types.h>
+#endif
+
+/* we know core's module_index is 0 */
+#undef APLOG_MODULE_INDEX
+#define APLOG_MODULE_INDEX AP_CORE_MODULE_INDEX
 
 typedef struct {
     const char *t_name;
@@ -583,7 +590,16 @@ static int log_tid(const ap_errorlog_info *info, const char *arg,
 {
 #if APR_HAS_THREADS
     int result;
-
+#endif
+#if HAVE_GETTID
+    if (arg && *arg == 'g') {
+        pid_t tid = syscall(SYS_gettid);
+        if (tid == -1)
+            return 0;
+        return apr_snprintf(buf, buflen, "%"APR_PID_T_FMT, tid);
+    }
+#endif
+#if APR_HAS_THREADS
     if (ap_mpm_query(AP_MPMQ_IS_THREADED, &result) == APR_SUCCESS
         && result != AP_MPMQ_NOT_SUPPORTED)
     {
@@ -638,8 +654,8 @@ static int log_log_id(const ap_errorlog_info *info, const char *arg,
             return cpystrn(buf, info->c->log_id, buflen);
         }
     }
-    else if (info->r) {
-        return cpystrn(buf, info->r->log_id, buflen);
+    else if (info->rmain) {
+        return cpystrn(buf, info->rmain->log_id, buflen);
     }
     return 0;
 }
@@ -1133,7 +1149,7 @@ static void log_error_core(const char *file, int line, int module_index,
 
         /* the faked server_rec from mod_cgid does not have s->module_config */
         if (s->module_config) {
-            sconf = ap_get_module_config(s->module_config, &core_module);
+            sconf = ap_get_core_module_config(s->module_config);
             if (c && !c->log_id) {
                 add_log_id(c, NULL);
                 if (sconf->error_log_conn && sconf->error_log_conn->nelts > 0)
@@ -1166,7 +1182,7 @@ static void log_error_core(const char *file, int line, int module_index,
     info.status        = 0;
     info.using_syslog  = (logf == NULL);
     info.startup       = ((level & APLOG_STARTUP) == APLOG_STARTUP);
-
+    info.format        = fmt;
 
     while (!done) {
         apr_array_header_t *log_format;
@@ -1361,6 +1377,25 @@ AP_DECLARE(void) ap_log_command_line(apr_pool_t *plog, server_rec *s)
                  "Command line: '%s'", result);
 }
 
+AP_DECLARE(void) ap_remove_pid(apr_pool_t *p, const char *rel_fname)
+{
+    apr_status_t rv;
+    const char *fname = ap_server_root_relative(p, rel_fname);
+
+    if (fname != NULL) {
+        rv = apr_file_remove(fname, p);
+        if (rv != APR_SUCCESS) {
+            ap_log_error(APLOG_MARK, APLOG_ERR, rv, ap_server_conf,
+                         "failed to remove PID file %s", fname);
+        }
+        else {
+            ap_log_error(APLOG_MARK, APLOG_INFO, 0, ap_server_conf,
+                         "removed PID file %s (pid=%" APR_PID_T_FMT ")",
+                         fname, getpid());
+        }
+    }
+}
+
 AP_DECLARE(void) ap_log_pid(apr_pool_t *p, const char *filename)
 {
     apr_file_t *pid_file = NULL;
