@@ -16,6 +16,8 @@
 
 #ifdef WIN32
 
+#include "apr.h"
+#include <process.h>
 #include "httpd.h"
 #include "http_main.h"
 #include "http_log.h"
@@ -41,14 +43,12 @@
 #include "apr_buckets.h"
 #include "scoreboard.h"
 
-#include <process.h>
-
 #ifdef __MINGW32__
 #include <mswsock.h>
 #endif 
 
 /*
- * The Windoes MPM uses a queue of completion contexts that it passes
+ * The Windows MPM uses a queue of completion contexts that it passes
  * between the accept threads and the worker threads. Declare the
  * functions to access the queue and the structures passed on the
  * queue in the header file to enable modules to access them
@@ -137,11 +137,12 @@ static void mpm_recycle_completion_context(winnt_conn_ctx_t *context)
     }
 }
 
-static winnt_conn_ctx_t *mpm_get_completion_context(void)
+static winnt_conn_ctx_t *mpm_get_completion_context(int *timeout)
 {
     apr_status_t rv;
     winnt_conn_ctx_t *context = NULL;
 
+    *timeout = 0;
     while (1) {
         /* Grab a context off the queue */
         apr_thread_mutex_lock(qlock);
@@ -165,7 +166,7 @@ static winnt_conn_ctx_t *mpm_get_completion_context(void)
                 /* All workers are busy, need to wait for one */
                 static int reported = 0;
                 if (!reported) {
-                    ap_log_error(APLOG_MARK, APLOG_WARNING, 0, ap_server_conf,
+                    ap_log_error(APLOG_MARK, APLOG_ERR, 0, ap_server_conf,
                                  "Server ran out of threads to serve "
                                  "requests. Consider raising the "
                                  "ThreadsPerChild setting");
@@ -180,8 +181,23 @@ static winnt_conn_ctx_t *mpm_get_completion_context(void)
                 rv = WaitForSingleObject(qwait_event, 1000);
                 if (rv == WAIT_OBJECT_0)
                     continue;
-                else /* Hopefully, WAIT_TIMEOUT */
+                else {
+                    if (rv == WAIT_TIMEOUT) {
+                        /* somewhat-normal condition where threads are busy */
+                        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, ap_server_conf,
+                                     "mpm_get_completion_context: Failed to get a "
+                                     "free context within 1 second");
+                        *timeout = 1;
+                    }
+                    else {
+                        /* should be the unexpected, generic WAIT_FAILED */
+                        ap_log_error(APLOG_MARK, APLOG_WARNING, apr_get_os_error(),
+                                     ap_server_conf,
+                                     "mpm_get_completion_context: "
+                                     "WaitForSingleObject failed to get free context");
+                    }
                     return NULL;
+                }
             } else {
                 /* Allocate another context.
                  * Note: Multiple failures in the next two steps will cause
@@ -277,8 +293,7 @@ static unsigned int __stdcall winnt_accept(void *lr_)
 #endif
     u_long zero = 0;
 
-    core_sconf = ap_get_module_config(ap_server_conf->module_config,
-                                      &core_module);
+    core_sconf = ap_get_core_module_config(ap_server_conf->module_config);
     accf_name = apr_table_get(core_sconf->accf_map, lr->protocol);
 
     if (strcmp(accf_name, "data") == 0)
@@ -338,28 +353,26 @@ reinit: /* target of data or connect upon too many AcceptEx failures */
     }
 
     ap_log_error(APLOG_MARK, APLOG_INFO, 0, ap_server_conf,
-                 "Child %d: Accept thread listening on %s:%d using %s", my_pid,
+                 "Child: Accept thread listening on %s:%d using AcceptFilter %s",
                  lr->bind_addr->hostname ? lr->bind_addr->hostname : "*",
                  lr->bind_addr->port, accf_name);
 
     while (!shutdown_in_progress) {
         if (!context) {
-            context = mpm_get_completion_context();
-            if (!context) {
-                /* Hopefully a temporary condition in the provider? */
-                ++err_count;
-                if (err_count > MAX_ACCEPTEX_ERR_COUNT) {
-                    ap_log_error(APLOG_MARK, APLOG_ERR, rv, ap_server_conf,
-                                 "winnt_accept: Too many failures grabbing a "
-                                 "connection ctx.  Aborting.");
-                    break;
-                }
+            int timeout;
 
-                /* Temporary resource constraint? */
-                ap_log_error(APLOG_MARK, APLOG_DEBUG, apr_get_netos_error(), 
-                             ap_server_conf,
-                             "winnt_accept: Failed to grab a connection ctx."
-                             "  Temporary resource constraint? Retrying.");
+            context = mpm_get_completion_context(&timeout);
+            if (!context) {
+                if (!timeout) {
+                    /* Hopefully a temporary condition in the provider? */
+                    ++err_count;
+                    if (err_count > MAX_ACCEPTEX_ERR_COUNT) {
+                        ap_log_error(APLOG_MARK, APLOG_CRIT, 0, ap_server_conf,
+                                     "winnt_accept: Too many failures grabbing a "
+                                     "connection ctx.  Aborting.");
+                        break;
+                    }
+                }
                 Sleep(100);
                 continue;
             }
@@ -443,11 +456,10 @@ reinit: /* target of data or connect upon too many AcceptEx failures */
                     ++err_count;
                     if (err_count > MAX_ACCEPTEX_ERR_COUNT) {
                         ap_log_error(APLOG_MARK, APLOG_ERR, rv, ap_server_conf,
-                                     "Child %d: Encountered too many AcceptEx "
+                                     "Child: Encountered too many AcceptEx "
                                      "faults accepting client connections. "
                                      "Possible causes: dynamic address renewal, "
-                                     "or incompatible VPN or firewall software. ",
-                                     my_pid);
+                                     "or incompatible VPN or firewall software. ");
                         ap_log_error(APLOG_MARK, APLOG_NOTICE, rv, ap_server_conf,
                                      "winnt_mpm: falling back to "
                                      "'AcceptFilter none'.");
@@ -465,9 +477,8 @@ reinit: /* target of data or connect upon too many AcceptEx failures */
                     ++err_count;
                     if (err_count > MAX_ACCEPTEX_ERR_COUNT) {
                         ap_log_error(APLOG_MARK, APLOG_ERR, rv, ap_server_conf,
-                                     "Child %d: Encountered too many AcceptEx "
-                                     "faults accepting client connections.",
-                                     my_pid);
+                                     "Child: Encountered too many AcceptEx "
+                                     "faults accepting client connections.");
                         ap_log_error(APLOG_MARK, APLOG_NOTICE, rv, ap_server_conf,
                                      "winnt_mpm: falling back to "
                                      "'AcceptFilter none'.");
@@ -556,7 +567,7 @@ reinit: /* target of data or connect upon too many AcceptEx failures */
             if (context->accept_socket != INVALID_SOCKET)
                 closesocket(context->accept_socket);
 
-            /* This could be a persistant event per-listener rather than
+            /* This could be a persistent event per-listener rather than
              * per-accept.  However, the event needs to be removed from
              * the target socket if not removed from the listen socket
              * prior to accept(), or the event select is inherited.
@@ -593,7 +604,7 @@ reinit: /* target of data or connect upon too many AcceptEx failures */
                     continue;
                 }
 
-                /* A more serious error that 'retry', log it */
+                /* A more serious error than 'retry', log it */
                 ap_log_error(APLOG_MARK, APLOG_WARNING,
                              rv, ap_server_conf,
                              "accept() failed.");
@@ -605,9 +616,8 @@ reinit: /* target of data or connect upon too many AcceptEx failures */
                     ++err_count;
                     if (err_count > MAX_ACCEPTEX_ERR_COUNT) {
                         ap_log_error(APLOG_MARK, APLOG_ERR, rv, ap_server_conf,
-                                     "Child %d: Encountered too many accept() "
-                                     "resource faults, aborting.",
-                                     my_pid);
+                                     "Child: Encountered too many accept() "
+                                     "resource faults, aborting.");
                         break;
                     }
                     continue;
@@ -658,7 +668,7 @@ reinit: /* target of data or connect upon too many AcceptEx failures */
     }
 
     ap_log_error(APLOG_MARK, APLOG_INFO, APR_SUCCESS, ap_server_conf,
-                 "Child %d: Accept thread exiting.", my_pid);
+                 "Child: Accept thread exiting.");
     return 0;
 }
 
@@ -687,8 +697,8 @@ static winnt_conn_ctx_t *winnt_get_connection(winnt_conn_ctx_t *context)
         if (!rc) {
             rc = apr_get_os_error();
             ap_log_error(APLOG_MARK, APLOG_DEBUG, rc, ap_server_conf,
-                         "Child %d: GetQueuedComplationStatus returned %d",
-                         my_pid, rc);
+                         "Child: GetQueuedComplationStatus returned %d",
+                         rc);
             continue;
         }
 
@@ -718,6 +728,8 @@ static winnt_conn_ctx_t *winnt_get_connection(winnt_conn_ctx_t *context)
  */
 static DWORD __stdcall worker_main(void *thread_num_val)
 {
+    apr_thread_t *thd = NULL;
+    apr_os_thread_t osthd;
     static int requests_this_child = 0;
     winnt_conn_ctx_t *context = NULL;
     int thread_num = (int)thread_num_val;
@@ -727,6 +739,9 @@ static DWORD __stdcall worker_main(void *thread_num_val)
     conn_rec *c;
     apr_int32_t disconnected;
 
+    osthd = apr_os_thread_current();
+    apr_os_thread_put(&thd, &osthd, pchild);
+
     while (1) {
 
         ap_update_child_status_from_indexes(0, thread_num, SERVER_READY, NULL);
@@ -763,6 +778,8 @@ static DWORD __stdcall worker_main(void *thread_num_val)
             continue;
         }
 
+        c->current_thread = thd;
+
         /* follow ap_process_connection(c, context->sock) logic
          * as it left us no chance to reinject our first data bucket.
          */
@@ -849,7 +866,7 @@ static void cleanup_thread(HANDLE *handles, int *thread_cnt,
  * monitors the child process for maintenance and shutdown
  * events.
  */
-static void create_listener_thread()
+static void create_listener_thread(void)
 {
     unsigned tid;
     int num_listeners = 0;
@@ -910,8 +927,7 @@ void child_main(apr_pool_t *pconf)
     max_requests_per_child_event = CreateEvent(NULL, TRUE, FALSE, NULL);
     if (!max_requests_per_child_event) {
         ap_log_error(APLOG_MARK, APLOG_CRIT, apr_get_os_error(), ap_server_conf,
-                     "Child %d: Failed to create a max_requests event.", 
-                     my_pid);
+                     "Child: Failed to create a max_requests event.");
         exit(APEXIT_CHILDINIT);
     }
     child_events[0] = exit_event;
@@ -925,12 +941,12 @@ void child_main(apr_pool_t *pconf)
     status = apr_proc_mutex_lock(start_mutex);
     if (status != APR_SUCCESS) {
         ap_log_error(APLOG_MARK, APLOG_ERR, status, ap_server_conf,
-                     "Child %d: Failed to acquire the start_mutex. "
-                     "Process will exit.", my_pid);
+                     "Child: Failed to acquire the start_mutex. "
+                     "Process will exit.");
         exit(APEXIT_CHILDINIT);
     }
     ap_log_error(APLOG_MARK, APLOG_NOTICE, APR_SUCCESS, ap_server_conf,
-                 "Child %d: Acquired the start mutex.", my_pid);
+                 "Child: Acquired the start mutex.");
 
     /*
      * Create the worker thread dispatch IOCompletionPort
@@ -943,7 +959,7 @@ void child_main(apr_pool_t *pconf)
     if (!qwait_event) {
         ap_log_error(APLOG_MARK, APLOG_CRIT, apr_get_os_error(), 
                      ap_server_conf,
-                     "Child %d: Failed to create a qwait event.", my_pid);
+                     "Child: Failed to create a qwait event.");
         exit(APEXIT_CHILDINIT);
     }
 
@@ -951,8 +967,7 @@ void child_main(apr_pool_t *pconf)
      * Create the pool of worker threads
      */
     ap_log_error(APLOG_MARK, APLOG_NOTICE, APR_SUCCESS, ap_server_conf,
-                 "Child %d: Starting %d worker threads.",
-                 my_pid, ap_threads_per_child);
+                 "Child: Starting %d worker threads.", ap_threads_per_child);
     child_handles = (HANDLE) apr_pcalloc(pchild, ap_threads_per_child
                                                   * sizeof(HANDLE));
     apr_thread_mutex_create(&child_lock, APR_THREAD_MUTEX_DEFAULT, pchild);
@@ -972,11 +987,11 @@ void child_main(apr_pool_t *pconf)
             if (child_handles[i] == 0) {
                 ap_log_error(APLOG_MARK, APLOG_CRIT, apr_get_os_error(),
                              ap_server_conf,
-                             "Child %d: CreateThread failed. Unable to "
+                             "Child: CreateThread failed. Unable to "
                              "create all worker threads. Created %d of the %d "
                              "threads requested with the ThreadsPerChild "
                              "configuration directive.",
-                             my_pid, threads_created, ap_threads_per_child);
+                             threads_created, ap_threads_per_child);
                 ap_signal_parent(SIGNAL_PARENT_SHUTDOWN);
                 goto shutdown;
             }
@@ -1048,15 +1063,14 @@ void child_main(apr_pool_t *pconf)
             /* Something serious is wrong */
             ap_log_error(APLOG_MARK, APLOG_CRIT, apr_get_os_error(),
                          ap_server_conf,
-                         "Child %d: WAIT_FAILED -- shutting down server", 
-                         my_pid);
+                         "Child: WAIT_FAILED -- shutting down server");
             break;
         }
         else if (cld == 0) {
             /* Exit event was signaled */
             ap_log_error(APLOG_MARK, APLOG_NOTICE, APR_SUCCESS, ap_server_conf,
-                         "Child %d: Exit event signaled. Child process is "
-                         "ending.", my_pid);
+                         "Child: Exit event signaled. Child process is "
+                         "ending.");
             break;
         }
         else {
@@ -1064,9 +1078,9 @@ void child_main(apr_pool_t *pconf)
              * Signal the parent to restart
              */
             ap_log_error(APLOG_MARK, APLOG_NOTICE, APR_SUCCESS, ap_server_conf,
-                         "Child %d: Process exiting because it reached "
+                         "Child: Process exiting because it reached "
                          "MaxConnectionsPerChild. Signaling the parent to "
-                         "restart a new child process.", my_pid);
+                         "restart a new child process.");
             ap_signal_parent(SIGNAL_PARENT_RESTART);
             break;
         }
@@ -1088,7 +1102,7 @@ void child_main(apr_pool_t *pconf)
         apr_socket_close(lr->sd);
     }
 
-    /* Shutdown listener threads and pending AcceptEx socksts
+    /* Shutdown listener threads and pending AcceptEx sockets
      * but allow the worker threads to continue consuming from
      * the queue of accepted connections.
      */
@@ -1105,11 +1119,11 @@ void child_main(apr_pool_t *pconf)
     rv = apr_proc_mutex_unlock(start_mutex);
     if (rv == APR_SUCCESS) {
         ap_log_error(APLOG_MARK, APLOG_NOTICE, rv, ap_server_conf,
-                     "Child %d: Released the start mutex", my_pid);
+                     "Child: Released the start mutex");
     }
     else {
         ap_log_error(APLOG_MARK, APLOG_ERR, rv, ap_server_conf,
-                     "Child %d: Failure releasing the start mutex", my_pid);
+                     "Child: Failure releasing the start mutex");
     }
 
     /* Shutdown the worker threads
@@ -1117,8 +1131,8 @@ void child_main(apr_pool_t *pconf)
      */
     while (g_blocked_threads > 0) {
         ap_log_error(APLOG_MARK, APLOG_INFO, APR_SUCCESS, ap_server_conf,
-                     "Child %d: %d threads blocked on the completion port",
-                     my_pid, g_blocked_threads);
+                     "Child: %d threads blocked on the completion port",
+                     g_blocked_threads);
         for (i=g_blocked_threads; i > 0; i--) {
             PostQueuedCompletionStatus(ThreadDispatchIOCP, 0, 
                                        IOCP_SHUTDOWN, NULL);
@@ -1134,7 +1148,7 @@ void child_main(apr_pool_t *pconf)
     }
     apr_thread_mutex_unlock(qlock);
 
-    /* Give busy threads a chance to service their connections,
+    /* Give busy threads a chance to service their connections
      * (no more than the global server timeout period which 
      * we track in msec remaining).
      */
@@ -1158,9 +1172,9 @@ void child_main(apr_pool_t *pconf)
             if ((time_remains % 30000) == 0) {
                 ap_log_error(APLOG_MARK, APLOG_NOTICE, APR_SUCCESS, 
                              ap_server_conf,
-                             "Child %d: Waiting %d more seconds "
+                             "Child: Waiting %d more seconds "
                              "for %d worker threads to finish.", 
-                             my_pid, time_remains / 1000, threads_created);
+                             time_remains / 1000, threads_created);
             }
             /* We'll poll from the top, 10 times per second */
             Sleep(100);
@@ -1202,8 +1216,8 @@ void child_main(apr_pool_t *pconf)
     /* Kill remaining threads off the hard way */
     if (threads_created) {
         ap_log_error(APLOG_MARK, APLOG_NOTICE, APR_SUCCESS, ap_server_conf,
-                     "Child %d: Terminating %d threads that failed to exit.",
-                     my_pid, threads_created);
+                     "Child: Terminating %d threads that failed to exit.",
+                     threads_created);
     }
     for (i = 0; i < threads_created; i++) {
         int *score_idx;
@@ -1216,7 +1230,7 @@ void child_main(apr_pool_t *pconf)
         }
     }
     ap_log_error(APLOG_MARK, APLOG_NOTICE, APR_SUCCESS, ap_server_conf,
-                 "Child %d: All worker threads have exited.", my_pid);
+                 "Child: All worker threads have exited.");
 
     apr_thread_mutex_destroy(child_lock);
     apr_thread_mutex_destroy(qlock);
