@@ -22,6 +22,7 @@
 
 #include "httpd.h"
 #include "http_config.h"
+#include "http_core.h"
 #include "http_log.h"
 #include "util_filter.h"
 
@@ -29,7 +30,7 @@
    so we depend on a global to hold the correct pool
 */
 #define FILTER_POOL     apr_hook_global_pool
-#include "apr_hooks.h"   /* for apr_hook_global_pool */
+#include "ap_hooks.h"   /* for apr_hook_global_pool */
 
 /*
 ** This macro returns true/false if a given filter should be inserted BEFORE
@@ -45,7 +46,9 @@
  * filter names to filters
  */
 
-APLOG_USE_MODULE(core);
+/* we know core's module_index is 0 */
+#undef APLOG_MODULE_INDEX
+#define APLOG_MODULE_INDEX AP_CORE_MODULE_INDEX
 
 typedef struct filter_trie_node filter_trie_node;
 
