@@ -306,6 +306,7 @@ apr_port_t port;        /* port number */
 char proxyhost[1024];   /* proxy host name */
 int proxyport = 0;      /* proxy port */
 const char *connecthost;
+const char *myhost;
 apr_port_t connectport;
 const char *gnuplot;          /* GNUplot file */
 const char *csvperc;          /* CSV Percentile file */
@@ -370,6 +371,7 @@ apr_pool_t *cntxt;
 
 apr_pollset_t *readbits;
 
+apr_sockaddr_t *mysa;
 apr_sockaddr_t *destsa;
 
 #ifdef NOT_ASCII
@@ -509,7 +511,6 @@ static int ssl_rand_choosenum(int l, int h)
 
 static void ssl_rand_seed(void)
 {
-    int nDone = 0;
     int n, l;
     time_t t;
     pid_t pid;
@@ -521,7 +522,6 @@ static void ssl_rand_seed(void)
     t = time(NULL);
     l = sizeof(time_t);
     RAND_seed((unsigned char *)&t, l);
-    nDone += l;
 
     /*
      * seed in the current process id (usually just 4 bytes)
@@ -529,14 +529,12 @@ static void ssl_rand_seed(void)
     pid = getpid();
     l = sizeof(pid_t);
     RAND_seed((unsigned char *)&pid, l);
-    nDone += l;
 
     /*
      * seed in some current state of the run-time stack (128 bytes)
      */
     n = ssl_rand_choosenum(0, sizeof(stackdata)-128-1);
     RAND_seed(stackdata+n, 128);
-    nDone += 128;
 }
 
 static int ssl_print_connection_info(BIO *bio, SSL *ssl)
@@ -1205,6 +1203,10 @@ static void start_connect(struct connection * c)
     apr_err("socket", rv);
     }
 
+    if ((rv = apr_socket_bind(c->aprsock, mysa)) != APR_SUCCESS) {
+        apr_err("bind", rv);
+    }
+
     c->pollfd.desc_type = APR_POLL_SOCKET;
     c->pollfd.desc.s = c->aprsock;
     c->pollfd.reqevents = 0;
@@ -1606,7 +1608,7 @@ static void test(void)
      * XXX: would be nice.
      */
     stats = calloc(requests, sizeof(struct data));
-    if (stats == NULL) {
+    if (stats == NULL || con == NULL) {
     	err("Cannot allocate memory for result statistics");
     }
 
@@ -1703,6 +1705,14 @@ static void test(void)
 #endif              /* NOT_ASCII */
 
     /* This only needs to be done once */
+    if ((rv = apr_sockaddr_info_get(&mysa, myhost, APR_UNSPEC, 0, 0, cntxt)) != APR_SUCCESS) {
+        char buf[120];
+        apr_snprintf(buf, sizeof(buf),
+                 "apr_sockaddr_info_get() for %s", myhost);
+        apr_err(buf, rv);
+    }
+
+    /* This too */
     if ((rv = apr_sockaddr_info_get(&destsa, connecthost, APR_UNSPEC, connectport, 0, cntxt))
        != APR_SUCCESS) {
         char buf[120];
@@ -1832,14 +1842,14 @@ static void test(void)
 static void copyright(void)
 {
     if (!use_html) {
-        printf("This is ApacheBench, Version %s\n", AP_AB_BASEREVISION " <$Revision: 1004962 $>");
+        printf("This is ApacheBench, Version %s\n", AP_AB_BASEREVISION " <$Revision: 1139530 $>");
         printf("Copyright 1996 Adam Twiss, Zeus Technology Ltd, http://www.zeustech.net/\n");
         printf("Licensed to The Apache Software Foundation, http://www.apache.org/\n");
         printf("\n");
     }
     else {
         printf("<p>\n");
-        printf(" This is ApacheBench, Version %s <i>&lt;%s&gt;</i><br>\n", AP_AB_BASEREVISION, "$Revision: 1004962 $");
+        printf(" This is ApacheBench, Version %s <i>&lt;%s&gt;</i><br>\n", AP_AB_BASEREVISION, "$Revision: 1139530 $");
         printf(" Copyright 1996 Adam Twiss, Zeus Technology Ltd, http://www.zeustech.net/<br>\n");
         printf(" Licensed to The Apache Software Foundation, http://www.apache.org/<br>\n");
         printf("</p>\n<p>\n");
@@ -1861,6 +1871,7 @@ static void usage(const char *progname)
     fprintf(stderr, "    -c concurrency  Number of multiple requests to make\n");
     fprintf(stderr, "    -t timelimit    Seconds to max. wait for responses\n");
     fprintf(stderr, "    -b windowsize   Size of TCP send/receive buffer, in bytes\n");
+    fprintf(stderr, "    -B address      Address to bind to when making outgoing connections\n");
     fprintf(stderr, "    -p postfile     File containing data to POST. Remember also to set -T\n");
     fprintf(stderr, "    -u putfile      File containing data to PUT. Remember also to set -T\n");
     fprintf(stderr, "    -T content-type Content-type header for POSTing, eg.\n");
@@ -1890,7 +1901,11 @@ static void usage(const char *progname)
     fprintf(stderr, "    -h              Display usage information (this message)\n");
 #ifdef USE_SSL
     fprintf(stderr, "    -Z ciphersuite  Specify SSL/TLS cipher suite (See openssl ciphers)\n");
+#ifndef OPENSSL_NO_SSL2
     fprintf(stderr, "    -f protocol     Specify SSL/TLS protocol (SSL2, SSL3, TLS1, or ALL)\n");
+#else
+    fprintf(stderr, "    -f protocol     Specify SSL/TLS protocol (SSL3, TLS1, or ALL)\n");
+#endif
 #endif
     exit(EINVAL);
 }
@@ -2051,8 +2066,10 @@ int main(int argc, const char * const argv[])
     }
 #endif
 
+    myhost = NULL; /* 0.0.0.0 or :: */
+
     apr_getopt_init(&opt, cntxt, argc, argv);
-    while ((status = apr_getopt(opt, "n:c:t:b:T:p:u:v:rkVhwix:y:z:C:H:P:A:g:X:de:Sq"
+    while ((status = apr_getopt(opt, "n:c:t:b:T:p:u:v:rkVhwix:y:z:C:H:P:A:g:X:de:SqB:"
 #ifdef USE_SSL
             "Z:f:"
 #endif
@@ -2096,7 +2113,7 @@ int main(int argc, const char * const argv[])
             case 'p':
                 if (method != NO_METH)
                     err("Cannot mix POST with other methods\n");
-                if ((status = open_postfile(opt_arg)) != APR_SUCCESS) {
+                if (open_postfile(opt_arg) != APR_SUCCESS) {
                     exit(1);
                 }
                 method = POST;
@@ -2105,7 +2122,7 @@ int main(int argc, const char * const argv[])
             case 'u':
                 if (method != NO_METH)
                     err("Cannot mix PUT with other methods\n");
-                if ((status = open_postfile(opt_arg)) != APR_SUCCESS) {
+                if (open_postfile(opt_arg) != APR_SUCCESS) {
                     exit(1);
                 }
                 method = PUT;
@@ -2212,6 +2229,9 @@ int main(int argc, const char * const argv[])
             case 'V':
                 copyright();
                 return 0;
+            case 'B':
+                myhost = apr_pstrdup(cntxt, opt_arg);
+                break;
 #ifdef USE_SSL
             case 'Z':
                 ssl_cipher = strdup(opt_arg);
@@ -2219,8 +2239,10 @@ int main(int argc, const char * const argv[])
             case 'f':
                 if (strncasecmp(opt_arg, "ALL", 3) == 0) {
                     meth = SSLv23_client_method();
+#ifndef OPENSSL_NO_SSL2
                 } else if (strncasecmp(opt_arg, "SSL2", 4) == 0) {
                     meth = SSLv2_client_method();
+#endif
                 } else if (strncasecmp(opt_arg, "SSL3", 4) == 0) {
                     meth = SSLv3_client_method();
                 } else if (strncasecmp(opt_arg, "TLS1", 4) == 0) {
